package Epoll::DB::Poll::Bdata;

# $Id$

use strict;
use warnings;
use base 'Epoll::DB::common';
use Epoll::DB::Poll;
use XML::Simple;
use Crypt::CBC;
use Crypt::DES_EDE3; # Called from CBC

=head1 NAME

Epoll::Model::Vote - Catalyst Model

=head1 DESCRIPTION

Catalyst Model.

=cut

sub new {
    my ($class, $dbstring, $poll) = @_;
    
    bless {
        poll => $poll,
        dbstring => $dbstring,
        db => Epoll::DB::common::_newdb($dbstring),
        data => {}
    }, $class;
}

sub voteid { $_[0]->poll->voteid }

sub poll {
    my ($self) = @_;
    $self->{poll};
}

sub uid {
    my ($self) = @_;
    $self->{data}{id} ||= Epoll::DB::common::gen_uid();
}

sub items {
    my ($self) = @_;
    @{$self->{data}{content} || []}
}

sub add_item {
    my ($self, @items) = @_;
    foreach (@items) {
        s/^\s+//;
        s/\s+$//;
        s/\s+/ /g;
        $_ = lc($_);

        push(@{$self->{data}{content}}, $_);
    }
}

sub xmlout {
    my ($self) = @_;
    my $xml = XML::Simple->new(ForceArray => 1, RootName => 'ballot');
    $xml->XMLout({
        id => $self->uid,
        content => [ $self->items ],
    });
}

sub xmlin {
    my ($self, $data) = @_;
    my $xmldata = XMLin($data, ForceArray => ['content']);
    $self->{data}{id} = $xmldata->{id};
    if ($xmldata->{content}) {
        if (ref $xmldata->{content}) {
            foreach (@{ $xmldata->{content} || []}) {
                $self->add_item($_);
            }
        } else {
            $self->add_item($xmldata->{content});
        }
    }
}

sub submit {
    my ($self, $voting, $referal) = @_;
    
    for (0..2) { # 3 try
        # First we register voting has voted
        $self->poll->_register_signing($voting, $referal) or return; # TODO error ?

        # registring choices
        if ($self->poll->is_crypted
            ? $self->_store_crypted()
            : $self->_store_clear()) {
            last;
        } else {
            $self->rollback;
            next;
        }

    }
        # everything went fine, saving!
    $self->commit;

    $self->uid
}

sub _store_clear {
    my ($self) = @_;
    my $addb = $self->db->prepare_cached(
        q{
        insert into ballot (id, poll, invalid) values (?,?,?)
        }
    );

    my $needvalid = 'f';
    my %items = ();
    foreach($self->items) {
        if ($self->poll->find_choice_key($_)) {
            $items{$_} = 't';
        } else {
            $items{$_} = 'f';
            $needvalid = undef;
        }
    }

    $addb->execute($self->uid, $self->voteid, $needvalid) or do {
        $self->rollback;
        return;
    };

    my $addbc = $self->db->prepare_cached(
        q{
        insert into ballot_item (id, value, fromlist) values (?,?,?)
        }
    );
    foreach (keys %items) {
        $_ or next;
        $addbc->execute($self->uid, $_, $items{$_}) or do {
            $self->rollback;
            return;
        };
    }

    1;
}

sub _store_crypted {
    my ($self) = @_;
    my $symkey = map{ chr(rand(256)) } (1 .. 24);
    my $cipher = new Crypt::CBC($symkey, 'DES_EDE3');
    my $encryptedballot = $cipher->encrypt_hex(
        $self->xmlout()
    );
    my $encsymkey = $self->poll->rsa->encrypt (
        Message    => $symkey,
        Key        => $self->poll->public_key,
        Armour     => 1,
    ) || die $self->poll->rsa->errstr();

    my $addenc = $self->db->prepare_cached(
        q{insert into ballot_enc (id, data, enckey, poll) values (?,?,?,?)}
    );

    my $uid = Epoll::DB::common::gen_uid();
    $addenc->execute($uid, $encryptedballot, $encsymkey, $self->voteid);
}

sub _load_crypted {
    my ($self, $ballotid, $privkey) = @_;
    my $sth = $self->db->prepare_cached(
        q{select * from ballot_enc where id = ? for update}
    );
    $sth->execute($ballotid);
    my $ballot = $sth->fetchrow_hashref;
    $sth->finish;
    my $encsymkey = $ballot->{enckey};
    my $data = $ballot->{data};
    my $symkey = $self->poll->rsa->decrypt (
        Cyphertext => $encsymkey,
        Key        => $privkey,
        Armour     => 1,
    ) || die $self->poll->rsa->errstr();
    my $cipher = new Crypt::CBC($symkey, 'DES_EDE3');
    $self->xmlin($cipher->decrypt_hex($data));
}

sub decrypt {
    my ($self, $ballotid, $privkey) = @_;
    $self->_load_crypted($ballotid, $privkey);
    $self->_store_clear or return;
    my $upd = $self->db->prepare_cached(q{update ballot_enc set decrypted = true where id = ?});
    if ($upd->execute($ballotid)) {
        $self->commit;
        return;
    } else {
        $self->rollback;
        return 1;
    }
}


=head1 AUTHOR

Thauvin Olivier

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself or CeCILL.

=cut

1;
